@@ -1,15 +1,18 @@
 package Net::DNS::Question;
 #
-# $Id: Question.pm 319 2005-05-30 17:12:09Z olaf $
+# $Id: Question.pm 673 2007-08-01 09:56:57Z olaf $
 #
 use strict;
-use bytes;
+BEGIN { 
+    eval { require bytes; }
+} 
+
 use vars qw($VERSION $AUTOLOAD);
 
 use Carp;
 use Net::DNS;
 
-$VERSION = (qw$LastChangedRevision: 319 $)[1];
+$VERSION = (qw$LastChangedRevision: 673 $)[1];
 
 =head1 NAME
 
@@ -33,41 +36,65 @@ question section of a DNS packet.
 Creates a question object from the domain, type, and class passed
 as arguments.
 
+RFC4291 and RFC4632 IP address/prefix notation is supported for
+queries in in-addr.arpa and ip6.arpa subdomains.
+
 =cut
 
 sub new {
 	my $class = shift;
-	my %self = (
-		"qname"		=> undef,
-		"qtype"		=> undef,
-		"qclass"	=> undef,
-	);
 
-	my ($qname, $qtype, $qclass) = @_;
+	my $qname = shift;
+	my $qtype = uc shift || 'A';
+	my $qclass = uc shift || 'IN';
 
-	$qname  = "" if !defined($qname);
+	$qname = '' unless defined $qname;	# || ''; is NOT same!
+	$qname =~ s/\.+$//o;	# strip gratuitous trailing dot
 
-	$qtype  = defined($qtype)  ? uc($qtype)  : "ANY";
-	$qclass = defined($qclass) ? uc($qclass) : "ANY";
+ 	# Check if the caller has the type and class reversed.
+ 	# We are not that kind for unknown types.... :-)
+	($qtype, $qclass) = ($qclass, $qtype)
+		if exists $Net::DNS::classesbyname{$qtype}
+		and exists $Net::DNS::typesbyname{$qclass};
 
-	# Check if the caller has the type and class reversed.
-	# We are not that kind for unknown types.... :-)
-	if ((!exists $Net::DNS::typesbyname{$qtype} ||
-	     !exists $Net::DNS::classesbyname{$qclass})
-	    && exists $Net::DNS::classesbyname{$qtype}
-	    && exists $Net::DNS::typesbyname{$qclass}) {
-
-		($qtype, $qclass) = ($qclass, $qtype);
+	# if argument is an IP address, do appropriate reverse lookup
+	my $reverse = dns_addr($qname) if $qname =~ m/\d$|[:\/]/o;
+	if ( $reverse ) {
+		$qname = $reverse;
+		$qtype = 'PTR' if $qtype =~ m/^(A|AAAA)$/o;
 	}
 
-	$qname =~ s/^\.+//;
-	$qname =~ s/\.+$//;
+	my $self = {	qname	=> $qname,
+			qtype	=> $qtype,
+			qclass	=> $qclass
+			};
 
-	$self{"qname"}  = $qname;
-	$self{"qtype"}  = $qtype;
-	$self{"qclass"} = $qclass;
+	bless $self, $class;
+}
 
-	bless \%self, $class;
+
+sub dns_addr {
+	my $arg = shift;	# name or IP address
+
+	# If arg looks like IP4 address then map to in-addr.arpa space
+	if ( $arg =~ /((^|\d+\.)+\d+)($|\/(\d*))/o ) {
+		my @parse = split /\./, $1;
+		my $prefx = $4 || @parse<<3;
+		my $last = $prefx > 24 ? 3 : ($prefx-1)>>3;
+		return join '.', reverse( (@parse,(0)x3)[0 .. $last] ), 'in-addr.arpa';
+	}
+
+	# If arg looks like IP6 address then map to ip6.arpa space
+	if ( $arg =~ /^((\w*:)+)(\w*)($|\/(\d*))/o ) {
+		my @parse = split /:/, (reverse "0${1}0${3}"), 9;
+		my @xpand = map{/^$/ ? ('0')x(9-@parse) : $_} @parse;
+		my $prefx = $5 || @xpand<<4;
+		my $hex = pack 'A4'x8, map{$_.'000'} ('0')x(8-@xpand), @xpand;
+		my $len = $prefx > 124 ? 32 : ($prefx+3)>>2;
+		return join '.', split(//, substr($hex,-$len) ), 'ip6.arpa';
+	}
+
+	return undef;
 }
 
 
@@ -106,26 +133,16 @@ known as C<zclass> and refers to the zone's class.
 =cut
 
 sub AUTOLOAD {
-	my ($self) = @_;
-	
+	my $self = shift;
+
 	my $name = $AUTOLOAD;
-	$name =~ s/.*://;
+	$name =~ s/.*://o;
 
 	Carp::croak "$name: no such method" unless exists $self->{$name};
 
-	no strict q/refs/;
-	
-	*{$AUTOLOAD} = sub {
-		my ($self, $new_val) = @_;
-		
-		if (defined $new_val) {
-			$self->{"$name"} = $new_val;
-		}
-		
-		return $self->{"$name"};
-	};
-	
-	goto &{$AUTOLOAD};	
+	return $self->{$name} unless @_;
+
+	$self->{$name} = shift;
 }
 
 
@@ -141,7 +158,7 @@ Prints the question record on the standard output.
 
 =cut
 
-sub print {	print $_[0]->string, "\n"; }
+sub print {	print shift->string, "\n"; }
 
 =head2 string
 
@@ -173,10 +190,10 @@ domain names.
 sub data {
 	my ($self, $packet, $offset) = @_;
 
-	my $data = $packet->dn_comp($self->{"qname"}, $offset);
+	my $data = $packet->dn_comp($self->{qname}, $offset);
 
-	$data .= pack("n", Net::DNS::typesbyname(uc($self->{"qtype"})));
-	$data .= pack("n", Net::DNS::classesbyname(uc($self->{"qclass"})));
+	$data .= pack 'n', Net::DNS::typesbyname(uc $self->{qtype});
+	$data .= pack 'n', Net::DNS::classesbyname(uc $self->{qclass});
 	
 	return $data;
 }
@@ -187,6 +204,8 @@ Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
+Portions Copyright (c) 2003,2006-2007 Dick Franks.
+
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
 
