@@ -1,6 +1,6 @@
 package Net::DNS::RR::NS;
 #
-# $Id: NS.pm 554 2006-01-19 12:51:30Z olaf $
+# $Id: NS.pm 632 2007-03-12 13:24:21Z olaf $
 #
 use strict;
 BEGIN { 
@@ -9,7 +9,7 @@ BEGIN {
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 554 $)[1];
+$VERSION = (qw$LastChangedRevision: 632 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -55,7 +55,7 @@ sub _canonicalRdata {
     # rdata contains a compressed domainname... we should not have that.
 	my ($self) = @_;
 	my $rdata;
-	$rdata=$self->_name2wire($self->{"nsdname"});
+	$rdata= $self->_name2wire(lc($self->{"nsdname"}));
 	return $rdata;
 }
 
