@@ -1,13 +1,15 @@
 package Net::DNS::RR::RP;
 #
-# $Id: RP.pm 319 2005-05-30 17:12:09Z olaf $
+# $Id: RP.pm 632 2007-03-12 13:24:21Z olaf $
 #
 use strict;
-use bytes;
+BEGIN { 
+    eval { require bytes; }
+} 
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 319 $)[1];
+$VERSION = (qw$LastChangedRevision: 632 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -57,8 +59,8 @@ sub _canonicalRdata {
     my $self  = shift;
     my $rdata = "";
 	if (exists $self->{"mbox"}) {
-		$rdata .= $self->_name2wire($self->{"mbox"});
-		$rdata .= $self->_name2wire($self->{"txtdname"});
+		$rdata .= $self->_name2wire(lc($self->{"mbox"}));
+		$rdata .= $self->_name2wire(lc($self->{"txtdname"}));
 
 
 	}
